@@ -1,7 +1,5 @@
 from typing import TYPE_CHECKING, Union
 
-import numpy as np
-
 if TYPE_CHECKING:
     import pandas
     from azure.kusto.data._models import KustoResultTable, KustoStreamingResultTable
@@ -30,11 +28,13 @@ def to_pandas_timedelta(raw_value: Union[int, float, str]) -> "pandas.Timedelta"
             return pd.to_timedelta(formatted_value)
 
 
-def dataframe_from_result_table(table: "Union[KustoResultTable, KustoStreamingResultTable]") -> "pandas.DataFrame":
+def dataframe_from_result_table(table: "Union[KustoResultTable, KustoStreamingResultTable]", nullable_bools: bool = False) -> "pandas.DataFrame":
     """Converts Kusto tables into pandas DataFrame.
     :param azure.kusto.data._models.KustoResultTable table: Table received from the response.
+    :param nullable_bools: When True, converts bools that are 'null' from kusto or 'None' from python to pandas.NA. This will be the default in the future.
     :return: pandas DataFrame.
     """
+    import numpy as np
     import pandas as pd
 
     if not table:
@@ -51,7 +51,7 @@ def dataframe_from_result_table(table: "Union[KustoResultTable, KustoStreamingRe
     # fix types
     for col in table.columns:
         if col.column_type == "bool":
-            frame[col.column_name] = frame[col.column_name].astype(bool)
+            frame[col.column_name] = frame[col.column_name].astype("boolean" if nullable_bools else bool)
         elif col.column_type == "int":
             frame[col.column_name] = frame[col.column_name].astype("Int32")
         elif col.column_type == "long":
@@ -65,3 +65,13 @@ def dataframe_from_result_table(table: "Union[KustoResultTable, KustoStreamingRe
             frame[col.column_name] = frame[col.column_name].apply(to_pandas_timedelta)
 
     return frame
+
+
+def get_string_tail_lower_case(val, length):
+    if length <= 0:
+        return ""
+
+    if length >= len(val):
+        return val.lower()
+
+    return val[len(val) - length :].lower()
