@@ -65,9 +65,9 @@ class KustoConnectionStringBuilder:
                 return cls.application_token
             if key in ["user token", "usertoken", "usrtoken"]:
                 return cls.user_token
-            if key in ["msi_auth"]:
+            if key in ["msi_auth", "msi authentication"]:
                 return cls.msi_auth
-            if key in ["msi_type"]:
+            if key in ["msi_type", "msi params"]:
                 return cls.msi_params
             if key in ["az cli"]:
                 return cls.az_cli
